#### Script to summarize Wikipedia Edit Histories
#### Accompanies article "Content volatility of politically controversial science topics in Wikipedia"
#### Script uses some LINUX programs (wdiff, curl, wget) that need to be installed and available to R as system() calls.

dir.create("complete")

url="http://toolserver.org/~daniel/WikiSense/Contributors.php"
library(lattice);library(multicore);library(reshape);library(XML)
library(xtable); library(multcomp)

### Build Table of topics and their categories
p=c("Acid_rain","Global_warming","Evolution","Continental_drift","Heliocentrism","General_relativity","Standard_Model")
p=factor(p,levels=p,ordered=T)
pt=c("Controversial","Controversial","Controversial",
     "Noncontroversial","Noncontroversial","Noncontroversial","Noncontroversial")


### Function to download the edit summaries from toolserver.org
gethistory=function(page,maxn=10000000){
  print(paste("Getting ",page))
  d=system(paste("curl ",url,
    " -s -G -d wikilang=en -d wikifam=.wikipedia.org -d page=",
    page," -d order=-edit_count -d max=",
    formatC(maxn,format="f",digits=0),
    " -d order=-edit_count -d format=tsv",sep=""),intern=T)
  d=do.call(rbind.data.frame,strsplit(d,split="\t"))#[,1:3]
  colnames(d)=c("date","user","desc")
  ## fix some metdata
  d$user=as.character(d$user)
  d$desc=as.character(d$desc)
  d$date=as.Date(d$date,"%Y-%m-%d %H:%M")
  d$week=as.numeric(format(d$date,"%U"))
  d$year=format(d$date,"%Y")
  ## add anonymous flag
  d$anon=grepl("anon",d$user)
  ## add reverted flag
  d$revert=grepl("Reverting|Reverted",d$desc)
  ## add page descriptor
  d$page=page
  ##########################################
  ## flag vandalism and correction of vandalism
  ## add revert vandalism flag
  d$revertvandalism=grepl("vandalism",d$desc)
  ## add bot flag
  d$bot=grepl("bot",d$user)
  ## get user IP if listed
  ips=regexpr("[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}",d$user)
  ips=cbind.data.frame(start=ips,count=attr(ips,"match.length"))
  d$userip=NA
  d$userip[ips$start!=-1]=substr(d$user[ips$start!=-1],ips$start[ips$start!=-1],
            ips$start[ips$start!=-1]+ips$count[ips$start!=-1]-1)
  ## get ip's listed in description
  ips=regexpr("[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}",d$desc)
  ips=cbind.data.frame(start=ips,count=attr(ips,"match.length"))
  d$vandalip=NA
  d$vandalip[ips$start!=-1]=substr(d$desc[ips$start!=-1],ips$start[ips$start!=-1],
              ips$start[ips$start!=-1]+ips$count[ips$start!=-1]-1)
  ## remove edits by bots and the 'vandalism' they identify
  dropvandal=F
  for(i in nrow(d):2) dropvandal[i]=ifelse(d$revertvandalism[i-1]&d$userip[i]==d$vandalip[i-1],T,F)
  d=d[!dropvandal,]
  ## add controversial flag
  d$cont=as.factor(pt[match(d$page,p)])
  ##########################################
  ## flag successional edits by same author
  successive=F
  for(i in 2:nrow(d)) successive[i]=ifelse(d$user[i]==d$user[i-1],T,F)
  ## Drop successive edits by the same user
  d=d[!successive,]
  ## sometimes there is a spurious /t that generates a separate column with missing colname,
  ## if present, delete it
  d=d[,!is.na(colnames(d))]
  return(d)
}

### Run it
d=do.call(rbind.data.frame,mclapply(p,gethistory))
### Save it for later
save(d,file="History.Rdata")

########################################
### Get complete edit history to summarize word changes
for(page in p)  {
  ## first get table of edit dates
  system(paste("curl -d \"&pages=",
               page,"&offset=1&limit=1000&action=submit\"",
               " http://en.wikipedia.org/w/index.php?title=Special:Export -o \"complete/",
               page,"_1.xml\"",sep=""))
  ## function to extract edit date-times from xml file
  getDates=function(file){
    h1=xmlTreeParse(file,useInternalNodes=T)
    ## get dates
    pdates=sapply(getNodeSet(h1,"/d:mediawiki/d:page/d:revision/d:timestamp",namespaces=c(d="http://www.mediawiki.org/xml/export-0.7/")),xmlValue)
    pdates=as.POSIXlt(gsub("T|Z"," ",pdates))
    return(pdates)
  }
  ## loop through dates until present and download edits 1000 at a time
  i=2  #start with second batch (first was downloaded above)
  maxdate=as.POSIXlt("1970-01-01 00:00:01")  #initial starting value
  while(maxdate<as.POSIXlt("2012-07-31 23:59:59")){
    dates=getDates(paste("complete/",page,"_",i-1,".xml",sep=""))  #get all dates in previously downloaded file
    if(length(dates)<1)  {maxdate=as.POSIXlt("2100-01-01 00:00:01") ; next }  #if file empty, move on...
    maxdate=max(dates,na.rm=T)  #get max of dates in this batch
    ## download the next batch of 1k edits
    system(paste("curl -d \"&pages=",page,"&offset=",format(maxdate,"%Y-%m-%dT%H:%M:%SZ"),"&limit=1000&action=submit\"",
                 " http://en.wikipedia.org/w/index.php?title=Special:Export -o \"complete/",page,"_",i,".xml\"",sep=""))
    i=i+1  #update counter
    print(paste("Finished ",page,"up to",maxdate))
  }
  ## End the downloading loop
  print(paste("###############################################      Finished",page))
}


## Function to extract edits
fword=function(file){
  print(file)
  ar=xmlTreeParse(file,useInternalNodes=T)
  art=sapply(getNodeSet(ar,"/d:mediawiki/d:page/d:revision/d:text",
                        namespaces=c(d="http://www.mediawiki.org/xml/export-0.7/")),xmlValue)
  if(length(art)==0) return(NULL)  #if it's empty, move on...
  pdates=as.POSIXlt(sapply(getNodeSet(ar,"/d:mediawiki/d:page/d:revision/d:timestamp",
                                      namespaces=c(d="http://www.mediawiki.org/xml/export-0.7/")),
                           xmlValue))
  revisionid=sapply(getNodeSet(ar,"/d:mediawiki/d:page/d:revision/d:id",
                               namespaces=c(d="http://www.mediawiki.org/xml/export-0.7/")),xmlValue)
  user=sapply(getNodeSet(ar,"/d:mediawiki/d:page/d:revision/d:contributor",
                         namespaces=c(d="http://www.mediawiki.org/xml/export-0.7/")),xmlValue)
  successive=rep(NA,length(user))
  for(i in 1:(length(user)-1)) successive[i]=ifelse(pdates[i+1]==pdates[i]&user[i+1]==user[i],T,F)

  return(data.frame(date=as.Date(pdates),
                    datetime=pdates,
                    page=p[do.call(c,lapply(p,function(x) grepl(x,file)))],
                    text=art,
                    revisionid=revisionid,
                    user=user,
                    successive=successive,
                    stringsAsFactors=F))
}

## load the data
wc=do.call(rbind.data.frame,
           mclapply(list.files("complete",full=T,pattern="xml$"),fword))

## identify temporary working directory
scratch=tempdir()


## Process the word diffs, this requires the "wdiff" program to be installed and available to R
## http://www.gnu.org/software/wdiff/

wdiff<-function(i,wc){
  if(i%in%pretty(1:nrow(wc),100))   print(paste(i,"out of",nrow(wc)))
  if(wc$page[i]!=wc$page[i-1]) return(NULL)
  write.table(wc$text[i-1],file=paste(scratch,wc$page[i],"_",i,"_a.txt",sep=""))
  write.table(wc$text[i],file=paste(scratch,wc$page[i],"_",i+1,"_b.txt",sep=""))
  wd=do.call(rbind.data.frame,strsplit(system(paste("wdiff -123si ",
    paste(scratch,wc$page[i],"_",i,"_a.txt",sep="")," ",
    paste(scratch,wc$page[i],"_",i+1,"_b.txt",sep=""),sep=""),intern=T)," "))[,c(2,5,6,9,10,13,14)]
  colnames(wd)=1:ncol(wd)
  wd=apply(wd,1:2,function(x) sub("%","",as.character(x),fixed=T))  #drop the %
  wd2=as.numeric(c(wd[2,1],wd[2,2],wd[2,3],wd[2,4],wd[2,5],wd[1,4],wd[1,5],wd[2,6],wd[2,7]))
  names(wd2)=c("count","common","pcommon","inserted","pinserted","deleted","pdeleted","changed","pchanged")
  wd2=as.data.frame(t(wd2))
  wd2$date=wc$date[i]
  wd2$page=wc$page[i]
  return(wd2)
}

## run it
wd=do.call(rbind.data.frame,mclapply(2:nrow(wc),wdiff,wc))

## add 0s for dates with no changes
alldates=expand.grid(page=p,date=seq(min(wd$date),max(wd$date),1))
wd=merge(wd,alldates,all.y=T)
wd$change=ifelse(is.na(wd$count),F,T)
wd[is.na(wd)]=0

## add controversial tag
wd$cont=NA
wd$cont=as.factor(pt[match(wd$page,p)])

## sort it for convenience
wd=wd[order(wd$page,wd$date),]


## melt it
wdl=melt(wd,id.vars=c("date","page","cont","change"))
## update factors
levels(wdl$variable)=c("Word Count","# of common words","% of words in common","# inserted words",
        "% of words inserted","# of deleted words","% of words deleted","# of changed words",
        "% of words changed","# changes","% changes")

## add more fields
wd$totalchange=wd$inserted+wd$deleted+wd$changed
wd$ptotalchange=(wd$inserted+wd$deleted+wd$changed)/wd$count


### Save data as csv's for upload to figshare
write.csv(d,"editcount.csv",row.names=F)
write.csv(wd,"wordschanged.csv",row.names=F)

