#### Script to summarize Wikipedia Edit Histories
#### Accompanies article "Content volatility of politically controversial science topics in Wikipedia"

url="http://toolserver.org/~daniel/WikiSense/Contributors.php"
library(reshape);library(xtable); library(nparcomp)

### Build Table of topics and their categories
p=c("Acid_rain","Global_warming","Evolution","Continental_drift","Heliocentrism","General_relativity","Standard_Model")
p=factor(p,levels=p,ordered=T)
pt=c("Controversial","Controversial","Controversial",
     "Noncontroversial","Noncontroversial","Noncontroversial","Noncontroversial")

########################################################################
### load the data
d=read.csv("editcount.csv",stringsAsFactors=F)
d$date=as.Date(d$date)

wd=read.csv("wordschanged.csv",stringsAsFactors=F)
wd$date=as.Date(wd$date)


## Summarize edits per day
dd=data.frame(cast(d,page+date+cont~.,value="user",fun=length))
colnames(dd)[grep("all",colnames(dd))]="count"
dd$change=T  # add flag that this was a real change
dates=seq(min(d$date,na.rm=T),max(d$date,na.rm=T),by=1)
dt=expand.grid(page=p,date=dates,count=0)
dt=dt[!paste(dt$page,dt$date)%in%paste(dd$page,dd$date),]
dt$cont=as.factor(pt[match(dt$page,p)])
dt$change=F  # add flag that this was no change
dd=rbind.data.frame(dt[,c("page","date","count","cont","change")],dd)
dd=dd[order(dd$page,dd$date),]

### Limit to a date range used in the paper
dd=dd[dd$date>=as.Date("2003-06-12")&dd$date<=as.Date("2012-06-30"),]
wd=wd[wd$date>=as.Date("2003-06-12")&wd$date<=as.Date("2012-06-30"),]


### confirm date range
tapply(wd$date,list(page=wd$page),range,na.rm=T)
tapply(dd$date,list(page=dd$page),range,na.rm=T)


### calculate sum of of word changes per day
dw=melt(tapply(wd$totalchange,list(date=format(wd$date,"%Y-%m-%d"),page=wd$page),sum,na.rm=T))
### drop days with no changes
dw=dw[dw$value!=0,]
### recreate ordered factor to line up with other data
dw$page=factor(dw$page,levels=p,ordered=T)
dw$date=as.Date(dw$date)

### draw calculate pairwise comparisons

##  analyze edit counts
m1=mctp(count~page,data=dd,type="Tukey",alternative="two.sided",
        asy.method="mult.t",info=T,plot.simci=T)

### look at mean words changed
m2=mctp(value~page,data=dw,type="Tukey",alternative="two.sided",
        asy.method="mult.t",info=T,plot.simci=T)


#### build table of pairwise comparisons to assess which pairs have significant differences.
sup=cbind.data.frame(
  Contrast=rownames(m1$Contrast),
  round(m1$Analysis,3),round(m2$Analysis,3))
colnames(sup)=paste(rep(c("ec","wc"),each=5),colnames(sup),sep=".")
sup=sup[,-1]
sup$ec.p.Value[sup$ec.p.Value<0.001]="<0.001"
sup$wc.p.Value[sup$wc.p.Value<0.001]="<0.001"
sup


## function for geometric mean
gmean=function(x,...) exp(mean(log(x+1),na.rm=T))-1
gsd=function(x,...) exp(sd(log(x+1),na.rm=T))

#### Build summary table
pagesummary=data.frame(
  ## max edits per day
  maxEditsPerDay=apply(tapply(d$date,list(d$page,d$date),length),1,max,na.rm=T),
  ## mean edits per day
  EditsPerDay=paste(round(tapply(dd$count,dd$page,gmean),1),"±",
                    round(tapply(dd$count,dd$page,gsd),1)," (",
                    tapply(dd$count,dd$page,length),")",sep=""),
  ## mean words changed per day
  WordChangesPerDay=paste(round(tapply(dw$value,dw$page,gmean),1),"±",
                          round(tapply(dw$value,dw$page,gsd),1),"  (",
                          tapply(dw$value,dw$page,length),")",sep="")
  )
pagesummary

print(xtable(pagesummary),"html",file="table.html")


## pageviews were added 'manually' using data served at http://toolserver.org/~emw/wikistats/ 


## Some summary information for the manuscript

## print n's
nrow(dd)
nrow(dw)

### acid rain stats
table(d$page)  #number of edits
table(d$anon)  #table of anonymous edits
table(d$anon)[2]/(table(d$anon)[1]+table(d$anon)[2])  # % anonymous
